use crate::module::direnv::{DirenvModuleArgs, DirenvNixFunction};

#[derive(Default, serde::Deserialize, serde::Serialize)]
pub struct DirenvModuleConfig {
    pub nix: Option<String>,
    pub nodejs: Option<bool>,
}

impl From<DirenvModuleConfig> for DirenvModuleArgs {
    fn from(cfg: DirenvModuleConfig) -> Self {
        DirenvModuleArgs {
            nix: cfg
                .nix
                .clone()
                .and_then(|v| match v.split_whitespace().next() {
                    Some("flake") => Some(DirenvNixFunction::Flake),
                    Some("shell") => Some(DirenvNixFunction::Shell),
                    _ => None,
                }),
            nix_args: cfg.nix.and_then(|v| match v.split_once(" ") {
                Some((_, args)) => Some(String::from(args)),
                None => None,
            }),
            nodejs: cfg.nodejs.unwrap_or_default(),
        }
    }
}
