use std::collections::HashMap;

use super::Module;

const NODEJS_PART: &str = "\
# nodejs
layout node";

pub enum DirenvNixFunction {
    Flake,
    Shell,
}
impl std::fmt::Display for DirenvNixFunction {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Self::Flake => f.write_str("flake"),
            Self::Shell => f.write_str("nix"),
        }
    }
}

pub struct DirenvModuleArgs {
    pub nix: Option<DirenvNixFunction>,
    pub nix_args: Option<String>,
    pub nodejs: bool,
}

fn make_envrc_content(args: DirenvModuleArgs) -> String {
    [
        args.nix.map(|function| {
            format!(
                "# nix\nuse {} {}",
                function,
                args.nix_args.unwrap_or_default()
            )
            .trim_end()
            .to_owned()
        }),
        args.nodejs.then_some(NODEJS_PART.to_string()),
    ]
    .iter()
    .flatten()
    .cloned()
    .collect::<Vec<_>>()
    .join("\n")
    .to_string()
}

pub struct DirenvModule;

impl Module for DirenvModule {
    type ModuleArgs = DirenvModuleArgs;

    fn make(&self, args: Self::ModuleArgs) -> HashMap<String, String> {
        HashMap::from([(String::from(".envrc"), make_envrc_content(args))])
    }
}
