use std::collections::HashMap;

use super::Module;

const EDITORS_PART: &str = "\
# editors
.idea/
.vscode/
*.swp";

const ENV_PART: &str = "\
# env
.env*
!env.example";

const DIRENV_PART: &str = "\
# direnv
.direnv
.envrc";

const RUST_PART: &str = "\
# rust
/target";

const NIX_PART: &str = "\
# nix
/result";

const NODEJS_PART: &str = "\
# nodejs
node_modules/";

pub struct GitIgnoreModuleArgs {
    pub editors: bool,
    pub env: bool,
    pub direnv: bool,
    pub rust: bool,
    pub nodejs: bool,
    pub nix: bool,
    pub custom: Option<String>,
}

fn make_gitignore_content(args: GitIgnoreModuleArgs) -> String {
    [
        args.editors.then_some(EDITORS_PART),
        args.env.then_some(ENV_PART),
        args.direnv.then_some(DIRENV_PART),
        args.rust.then_some(RUST_PART),
        args.nodejs.then_some(NODEJS_PART),
        args.nix.then_some(NIX_PART),
        args.custom.map(|c| format!("# custom\n{}", c)).as_deref(),
    ]
    .iter()
    .flatten()
    .cloned()
    .collect::<Vec<_>>()
    .join("\n")
    .to_string()
}

pub struct GitIgnoreModule;

impl Module for GitIgnoreModule {
    type ModuleArgs = GitIgnoreModuleArgs;

    fn make(&self, args: Self::ModuleArgs) -> HashMap<String, String> {
        HashMap::from([(String::from(".gitignore"), make_gitignore_content(args))])
    }
}
