use std::collections::HashMap;
use std::io::{self, Write};

use super::Module;

pub struct NixFlakeModuleArgs {
    pub description: Option<String>,
    pub extra_inputs: Option<String>,
    pub extra_outputs: Option<String>,
}

fn make_flakenix_content(args: NixFlakeModuleArgs) -> Result<String, io::Error> {
    let mut s = Vec::new();
    writeln!(s, "{{")?;
    #[rustfmt::skip]
    writeln!(s, "  description = \"{}\";", args.description.unwrap_or_default())?;

    // ----------------------
    // INPUTS
    writeln!(s, "  inputs = {{")?;
    if let Some(extra_inputs) = args.extra_inputs {
        writeln!(s, "{}", extra_inputs.trim())?;
    }
    writeln!(s, "  }};")?;

    // ----------------------
    // OUTPUTS
    writeln!(s, "  outputs =")?;
    writeln!(s, "    {{ self")?;
    writeln!(s, "    , nixpkgs")?;
    writeln!(s, "    , ...")?;
    writeln!(s, "    }} @ inputs:")?;

    if let Some(extra_outputs) = args.extra_outputs {
        write!(s, "    ({});", extra_outputs.trim())?;
    } else {
        writeln!(s, "    {{")?;
        writeln!(s, "    }};")?;
    }

    writeln!(s, "}}")?;

    Ok(String::from_utf8(s).unwrap())
}

pub struct NixFlakeModule;

impl Module for NixFlakeModule {
    type ModuleArgs = NixFlakeModuleArgs;

    fn make(&self, args: Self::ModuleArgs) -> HashMap<String, String> {
        HashMap::from([(
            String::from("flake.nix"),
            make_flakenix_content(args).unwrap(),
        )])
    }
}
